@@ -1034,8 +1034,13 @@ class tube(Timeout, Logger):
             b'hooray_data'
             >>> context.clear()
         """
+        cached_data = self.buffer.get()
+        if cached_data and not self.isEnabledFor(logging.DEBUG):
+            with context.local(log_level='debug'):
+                self.debug('Received %#x bytes:' % len(cached_data))
+                self.maybe_hexdump(cached_data, level=logging.DEBUG)
         with context.local(log_level='debug'):
-            return self.clean(timeout)
+            return cached_data + self.clean(timeout)
 
     def connect_input(self, other):
         """connect_input(other)
